#include "SS_Display_Dwin.h"

SS_Display_Dwin::SS_Display_Dwin(SoftwareSerial& serial,uint32_t bps) : 
          _serial(serial),_bps(bps){
          _serial.begin(_bps);
          }
		  
//color1: front color, color2: background color
void SS_Display_Dwin::SET_Color(uint16_t color1,uint16_t color2)
{
    
     _serial.write(Start_Byte);
     _serial.write(CMD_SET_COLOR);
     _serial.write((uint8_t)(color1>>8));
     _serial.write((uint8_t)(color1>>0));
     _serial.write((uint8_t)(color2>>8));
     _serial.write((uint8_t)(color2>>0));
      writeEndByte();// 0xCC,0x33,0xC3,0x3C
}

//print a picture saved at the position Pic_No
void SS_Display_Dwin::FullPicture (uint8_t Pic_No) {
     _serial.write(Start_Byte);
     _serial.write(CMD_FullPic);
     _serial.write(Pic_No); 
     writeEndByte();
}

//print the line in position (x1,y1; x2,y2)
void SS_Display_Dwin::Line (uint16_t x1,uint16_t y1,uint16_t x2,uint16_t y2,uint16_t frontcolor, uint16_t background){
     
    SET_Color(frontcolor, background);
     _serial.write(Start_Byte);
     _serial.write(CMD_Line_FR_Color);
     _serial.write((uint8_t)(x1>>8));
     _serial.write((uint8_t)(x1>>0));
     _serial.write((uint8_t)(y1>>8));
     _serial.write((uint8_t)(y1>>0));
     _serial.write((uint8_t)(x2>>8));
     _serial.write((uint8_t)(x2>>0));
     _serial.write((uint8_t)(y2>>8));
     _serial.write((uint8_t)(y2>>0));
      writeEndByte();
}

//print a circle
void SS_Display_Dwin::Circle(uint8_t type,uint16_t x1,uint16_t y1,uint16_t r1,uint16_t color){
   
     SET_Color(color,color);
     _serial.write(Start_Byte);
     _serial.write(CMD_Circle);
     _serial.write(type);
     _serial.write((uint8_t)(x1>>8));
     _serial.write((uint8_t)(x1>>0));
     _serial.write((uint8_t)(y1>>8));
     _serial.write((uint8_t)(y1>>0));
     _serial.write((uint8_t)(r1>>0));
     writeEndByte();

}

//print the rectangle in position (x1,y1; x2,y2)
void SS_Display_Dwin::Rectangle (uint16_t x1,uint16_t y1,uint16_t x2,uint16_t y2,uint16_t frontcolor, uint16_t backgroundr){
    
     SET_Color(frontcolor, backgroundr);
     _serial.write(Start_Byte);
     _serial.write(CMD_Rectangle);
     _serial.write((uint8_t)(x1>>8));
     _serial.write((uint8_t)(x1>>0));
     _serial.write((uint8_t)(y1>>8));
     _serial.write((uint8_t)(y1>>0));
     _serial.write((uint8_t)(x2>>8));
     _serial.write((uint8_t)(x2>>0));
     _serial.write((uint8_t)(y2>>8));
     _serial.write((uint8_t)(y2>>0));
     writeEndByte();
}

//clean the screen
void SS_Display_Dwin::ClearScreen (uint16_t color){
     SET_Color(color, color);
     _serial.write(Start_Byte);
     _serial.write(CMD_Clear_Screen);
     writeEndByte();
}  

//paint an rectagle area with a defined color
void  SS_Display_Dwin::FullAreaColor (uint16_t x1,uint16_t y1,uint16_t x2,uint16_t y2,uint16_t color){
     SET_Color(color,color);
     _serial.write(Start_Byte);
     _serial.write(CMD_Full_Area_Front);
     _serial.write((uint8_t)(x1>>8));
     _serial.write((uint8_t)(x1>>0));
     _serial.write((uint8_t)(y1>>8));
     _serial.write((uint8_t)(y1>>0));
     _serial.write((uint8_t)(x2>>8));
     _serial.write((uint8_t)(x2>>0));
     _serial.write((uint8_t)(y2>>8));
     _serial.write((uint8_t)(y2>>0));
     writeEndByte();
}

/*
Pic_No: Picntur stroge position
*                  - x1: Area left-up X position of Picntur stroge position
*                  - y1: Area left-up Y position of Picntur stroge position
*                  - x2: Area right-down X position of Picntur stroge position
*                  - y2: Area right-down X position of Picntur stroge position
*                  - X: left-up X position of the Dispaly position
*                  - Y: left-up Y position of the Dispaly position
*/
void SS_Display_Dwin::Area_Picture_Rectangle (uint16_t Pic_No ,uint16_t x1,uint16_t y1,uint16_t x2, uint16_t y2,uint16_t X,uint16_t Y){

     _serial.write(Start_Byte);
     _serial.write(CMD_AreaPic);
     _serial.write(Pic_No);
     _serial.write((uint8_t)(x1>>8));
     _serial.write((uint8_t)(x1>>0));
     _serial.write((uint8_t)(y1>>8));
     _serial.write((uint8_t)(y1>>0));
     _serial.write((uint8_t)(x2>>8));
     _serial.write((uint8_t)(x2>>0));
     _serial.write((uint8_t)(y2>>8));
     _serial.write((uint8_t)(y2>>0));
     _serial.write((uint8_t)(X>>8));
     _serial.write((uint8_t)(X>>0));
     _serial.write((uint8_t)(Y>>8));
     _serial.write((uint8_t)(Y>>0));
     writeEndByte();// 0xCC,0x33,0xC3,0x3C

}
 
//write a string, color is fixed, there are 5 different type of font
void SS_Display_Dwin::Print_String(uint8_t Font_Size,uint16_t X,uint16_t Y,uint8_t *String){

    
     _serial.write(Start_Byte);

     switch (Font_Size)

     {
       case (0):
           _serial.write(0x53) ;  // Font 8x8
       break;

       case (1):
           _serial.write(0x6E) ;  // Font 12x12
       break;

       case (2):
           _serial.write(0x54) ;  // Font 16x16
       break;

       case (3):
           _serial.write(0x6F) ;  // Font 24x24
       break;

       case (4):
           _serial.write(0x55) ;  // Font 32x32
       break;

     }
     
     _serial.write((uint8_t)(X>>8));
     _serial.write((uint8_t)(X>>0));
     _serial.write((uint8_t)(Y>>8));
     _serial.write((uint8_t)(Y>>0));
     

     while(*String) //
      {
       _serial.write(*String);
       String++;
       }
       
      writeEndByte();
}

//draw a pixel in position x, y
void SS_Display_Dwin::Put_Pixel(uint16_t x,uint16_t y,uint16_t color){
  
     SET_Color(color,color);
     _serial.write(Start_Byte);
     _serial.write(0x51);
     _serial.write((uint8_t)(x>>8));
     _serial.write((uint8_t)(x>>0));
     _serial.write((uint8_t)(y>>8));
     _serial.write((uint8_t)(y>>0));
     writeEndByte();// 0xCC,0x33,0xC3,0x3C
}

//trasforma in esadecimale il numero passato con DAT
void SS_Display_Dwin::Print_DAT(uint16_t X,uint16_t Y,uint8_t font, uint16_t DAT){
	SET_Color(0xff7f,0xfffff);
    uint8_t Temp;
    uint8_t HexValue[]="0123456789ABCDEF";
    uint8_t DAT16[]="XXXX";
    uint8_t *TEMP;
    
    TEMP=DAT16;
    
     *TEMP=HexValue[(DAT&0xF000)>>12];
     *(TEMP+1)=HexValue[(DAT&0x0F00)>>8];
     *(TEMP+2)=HexValue[(DAT&0x00F0)>>4];
     *(TEMP+3)=HexValue[(DAT&0x000F)>>0];
                 
    Print_String( font ,X, Y,  TEMP);
}
//do the negative color of the selected area
void SS_Display_Dwin::ReverseAreaColor (uint16_t x1,uint16_t y1,uint16_t x2,uint16_t y2){
 
     _serial.write(Start_Byte);
     _serial.write(CMD_Reverse_Area_Color);
     _serial.write((uint8_t)(x1>>8));
     _serial.write((uint8_t)(x1>>0));
     _serial.write((uint8_t)(y1>>8));
     _serial.write((uint8_t)(y1>>0));
     _serial.write((uint8_t)(x2>>8));
     _serial.write((uint8_t)(x2>>0));
     _serial.write((uint8_t)(y2>>8));
     _serial.write((uint8_t)(y2>>0));
     writeEndByte();
}

void SS_Display_Dwin::writeEndByte() {
  //_serial.write(endByte, 4);
  _serial.write(endByte[0]);//0xCC
  _serial.write(endByte[1]);//0x33
  _serial.write(endByte[2]);//0xC3
  _serial.write(endByte[3]);//0x3C
}
